@@ -45,13 +45,13 @@ use ChemOnomatopist::Name::Part::Stem;
 use ChemOnomatopist::Util qw(
     all_max
     all_min
+    atomic_number
     cmp_arrays
     copy
+    element
     zip
 );
 use ChemOnomatopist::Util::Graph qw(
-    BFS_calculate_chain_length
-    BFS_is_chain_branched
     cyclic_components
     graph_center
     graph_cycle_core
@@ -60,6 +60,7 @@ use ChemOnomatopist::Util::Graph qw(
     graph_longest_paths_from_vertex
     graph_path_between_vertices
     graph_without_edge_attributes
+    neighbours_at_distance
     subgraph
     tree_branch_positions
     tree_number_of_branches
@@ -71,7 +72,7 @@ use Chemistry::OpenSMILES qw(
     is_single_bond
     is_triple_bond
 );
-use Graph::MoreUtils qw( SSSR graph_replace );
+use Graph::MoreUtils qw( SSSR );
 use Graph::Nauty qw( are_isomorphic );
 use Graph::Traversal::DFS;
 use Graph::Undirected;
@@ -94,7 +95,7 @@ sub get_name
         $graph = ChemOnomatopist::MolecularGraph->new( $what );
     } else {
         # Assume SMILES string
-        die "cannot handle stereochemistry now\n" if $what =~ /[\\\/@]/;
+        die "cannot handle stereochemistry now\n" if $CAUTIOUS && $what =~ /[\\\/@]/;
 
         require Chemistry::OpenSMILES::Parser;
         my $parser = Chemistry::OpenSMILES::Parser->new;
@@ -229,7 +230,7 @@ sub get_sidechain_name
     }
 
     # Collecting names of all the attachments
-    my $name = ChemOnomatopist::Name->new;
+    my $name = $chain->stereodescriptor_part;
     my @order = sort { $a cmp $b } keys %attachments;
     for my $i (0..$#order) {
         my $attachment_name = $order[$i];
@@ -417,7 +418,7 @@ sub get_mainchain_name
 
     # Collecting names of all the attachments
     my @order = sort { cmp_only_alphabetical( $a, $b ) || $a cmp $b } keys %attachments;
-    my $name = ChemOnomatopist::Name->new;
+    my $name = $chain->stereodescriptor_part;
     for my $i (0..$#order) {
         my $attachment_name = $order[$i];
         my( $attachment, @positions ) = @{$attachments{$attachment_name}};
@@ -605,7 +606,8 @@ sub find_groups
         $valence += $atom->{hcount} if $atom->{hcount};
         for my $neighbour ($graph->neighbours( $atom )) {
             my $order = 1;
-            if( $graph->has_edge_attribute( $atom, $neighbour, 'bond' ) ) {
+            if( $graph->has_edge_attribute( $atom, $neighbour, 'bond' ) &&
+                exists   $bond_symbol_to_order{$graph->get_edge_attribute( $atom, $neighbour, 'bond' )} ) {
                 $order = $bond_symbol_to_order{$graph->get_edge_attribute( $atom, $neighbour, 'bond' )};
             }
             $valence += $order;
@@ -622,7 +624,8 @@ sub find_groups
         my %valences;
         for my $edge ($core->edges) {
             my $order = 1;
-            if( $core->has_edge_attribute( @$edge, 'bond' ) ) {
+            if( $core->has_edge_attribute( @$edge, 'bond' ) &&
+                exists   $bond_symbol_to_order{$core->get_edge_attribute( @$edge, 'bond' )} ) {
                 $order = $bond_symbol_to_order{$core->get_edge_attribute( @$edge, 'bond' )};
             }
             $valences{$edge->[0]} += $order;
@@ -699,13 +702,13 @@ sub find_groups
                 (any  { !$_->is_hydrocarbon } @cycles) &&
                 are_isomorphic( graph_without_edge_attributes( $core ),
                                 ChemOnomatopist::Chain::Xanthene->ideal_graph,
-                                sub { return 'C' } ) ) {
+                                sub { 'C' } ) ) {
                 $compound = ChemOnomatopist::Chain::Xanthene->new( $graph, @cycles );
             } elsif( ChemOnomatopist::Chain::Aceylene->has_form( $core ) ) {
                 $compound = ChemOnomatopist::Chain::Aceylene->new( $graph, @cycles );
             } elsif( @cycles == 3 &&
-                     (grep { $_->is_benzene } @cycles) == 2 &&
-                     (grep { $_->is_hydrocarbon && $_->length == 5 } @cycles) &&
+                     (grep { $_->is_benzene }  @cycles) == 2 &&
+                     (any  { $_->length == 5 } @cycles) &&
                      ChemOnomatopist::Chain::Fluorene->has_form( $core ) ) {
                 $compound = ChemOnomatopist::Chain::Fluorene->new( $graph, @cycles );
             } elsif( @cycles >= 3 &&
@@ -716,7 +719,7 @@ sub find_groups
                      (all { $_->length == 6 } @cycles) &&
                      are_isomorphic( graph_without_edge_attributes( $core ),
                                      ChemOnomatopist::Chain::Phenanthrene->ideal_graph,
-                                     sub { return 'C' } ) ) {
+                                     sub { 'C' } ) ) {
                 $compound = ChemOnomatopist::Chain::Phenanthrene->new( $graph, @cycles );
             } elsif( @cycles >= 4 &&
                      (all { $_->length == 6 && $_->is_hydrocarbon } @cycles) &&
@@ -771,24 +774,6 @@ sub find_groups
     return;
 }
 
-# Derive the chemical element of atom or group representation
-sub element
-{
-    my( $atom_or_group ) = @_;
-    return undef unless ref $atom_or_group;
-
-    if( !blessed $atom_or_group ) {
-        die "unknown value '$atom_or_group' given for element()\n" unless ref $atom_or_group eq 'HASH';
-        return ucfirst $atom_or_group->{symbol};
-    }
-
-    if( $atom_or_group->isa( 'Chemistry::Atom' ) ) { # PerlMol Atom
-        return $atom_or_group->symbol;
-    }
-
-    return $atom_or_group->element;
-}
-
 sub charge
 {
     my( $atom_or_group ) = @_;
@@ -912,6 +897,8 @@ sub select_mainchain
         } elsif( @parents == 1 ) {
             if( blessed $parents[0] && $parents[0]->can( 'candidates' ) ) {
                 @chains = $parents[0]->candidates;
+            } elsif( blessed $parents[0] && !$parents[0]->is_part_of_chain ) {
+                @chains = ( ChemOnomatopist::Chain->new( $graph, undef, @parents ) );
             } else {
                 # As the starting position is known, it is enough to take the "sidechain"
                 # containing this particular parent:
@@ -1087,7 +1074,8 @@ sub select_mainchain
     if( $most_senior_group &&
         element( $groups[0] ) && element( $groups[0] ) eq 'C' &&
         !$chain->isa( ChemOnomatopist::Chain::Circular:: ) &&
-         $chain->number_of_groups( $most_senior_group ) ) {
+         $chain->number_of_groups( $most_senior_group ) &&
+         @vertices >= 3 ) {
 
         shift @vertices;
         pop @vertices;
@@ -1192,7 +1180,7 @@ sub select_sidechain
         return @chains_now;
     };
 
-    for my $rule ( sub { return @_ },
+    for my $rule ( sub { @_ },
                    \&rule_longest_chains,
                    \&rule_greatest_number_of_side_chains, # After this rule we are left with a set of longest chains all having the same number of side chains
                    $rule_lowest_free_valence,
@@ -1570,7 +1558,17 @@ sub cmp_heteroatom_seniority
 sub cmp_locants($$)
 {
     my( $A, $B ) = @_;
+
     return $A <=> $B if $A =~ /^[0-9]+$/ && $B =~ /^[0-9]+$/;
+
+    if( $A =~ /^[0-9]+/ && $B =~ /^[0-9]+/ ) {
+        $A =~ /^([0-9]+)/;
+        my $A_number = $1;
+        $B =~ /^([0-9]+)/;
+        my $B_number = $1;
+        return $A_number <=> $B_number if $A_number <=> $B_number;
+    }
+
     return $A cmp $B if $A !~ /^[0-9]+$/ && $B !~ /^[0-9]+$/;
     return ($A =~ /^[0-9]+$/) <=> ($B =~ /^[0-9]+$/); # Letters go first
 }
@@ -1681,4 +1679,30 @@ sub alkane_chain_name($)
     return IUPAC_numerical_multiplier( $N );
 }
 
+# Implemented according to BBv3 P-92.1.4
+sub order_by_neighbours($$$$)
+{
+    my( $graph, $parent, $A, $B ) = @_;
+
+    my $distance = 0;
+    my @frontA = ( $A );
+    my @frontB = ( $B );
+    while( @frontA || @frontB ) {
+        my $cmp = cmp_arrays( [ reverse sort map { atomic_number( $_ ) } @frontB ],
+                              [ reverse sort map { atomic_number( $_ ) } @frontA ] );
+        return $cmp if $cmp;
+
+        # BBv3 P-92.3: higher atomic numbers appear first
+        $cmp = cmp_arrays( [ reverse sort map { exists $_->{isotope} ? $_->{isotope} : atomic_number( $_ ) } @frontB ],
+                           [ reverse sort map { exists $_->{isotope} ? $_->{isotope} : atomic_number( $_ ) } @frontA ] );
+        return $cmp if $cmp;
+
+        @frontA = neighbours_at_distance( $graph, $A, $parent, $distance, set( $parent ) );
+        @frontB = neighbours_at_distance( $graph, $B, $parent, $distance, set( $parent ) );
+        $distance++;
+    }
+
+    return 0;
+}
+
 1;
