@@ -6,7 +6,6 @@ package ChemOnomatopist::Chain::Bicycle;
 use strict;
 use warnings;
 
-use ChemOnomatopist;
 use ChemOnomatopist::Chain::Circular;
 use ChemOnomatopist::Chain::Bicycle::Purine;
 use ChemOnomatopist::Chain::Monocycle;
@@ -36,7 +35,6 @@ our @names = (
 
     [ qw( NCCNCC CCCCCC quinoxaline ) ],
 
-    [ 'c:c:c:n:c:c:', 'n:c:c:c:c:c:', '1,5-naphthyridine' ], # TODO: There are isomers
     [ 'c:n:n:c:c:c:', 'c:c:c:c:c:c:', 'phthalazine' ],
     [ 'n:c:c:c:c:c:', 'c:c:c:c:c:c:', 'quinoline' ],
 
@@ -93,7 +91,7 @@ sub new
     # Has to be created early to be given for fused parts
     my $self = bless { graph => $graph, vertices => \@vertices }, $class;
 
-    my $subgraph = $graph->subgraph( \@vertices );
+    my $subgraph = $graph->subgraph( @vertices );
     my @bridge = grep { $subgraph->degree( $_ ) == 3 } @vertices;
     $subgraph->delete_edge( @bridge );
     @vertices = Graph::Traversal::DFS->new( $subgraph, start => $bridge[0] )->dfs;
@@ -107,7 +105,7 @@ sub new
             die "bicycles with three and four-membered cycles are not supported yet\n"
         }
 
-        my $subgraph = $graph->subgraph( [ @{$components[$_]}, @bridge ] );
+        my $subgraph = $graph->subgraph( @{$components[$_]}, @bridge );
         $subgraph->delete_edge( @bridge );
         my @path = Graph::Traversal::DFS->new( $subgraph, start => $bridge[$_] )->dfs;
         push @path, shift @path;
@@ -170,18 +168,24 @@ sub new
                         $self->flipped_horizontally->flipped_vertically );
 
         # Establish the order
+        # FIXME: Somehow the rules from BBv3 P-14.4 interplay here
         for my $rule (
-                       # P-25.3.3.1.2 (a): Lower locants for heteroatoms
-                       \&ChemOnomatopist::rule_lowest_numbered_heteroatoms,
-                       # P-25.3.3.1.2 (b): Lower locants for senior heteroatoms
-                       \&ChemOnomatopist::rule_lowest_numbered_most_senior_heteroatoms,
-                       # P-25.3.3.1.2 (c): Lower locants for fusion carbon atoms
-                       \&rule_lowest_numbered_fusion_carbons,
-                       # P-25.3.3.1.2 (d): Lower locants for fusion heteroatoms (rather than nonfusion)
-                       \&rule_lowest_numbered_fusion_heteroatoms,
-                       # TODO: P-25.3.3.1.2 (e): Lower locants for interior heteroatom
-                       # P-25.3.3.1.2 (f): Lower locants for indicated hydrogen atoms
-                       \&ChemOnomatopist::rule_lowest_numbered_indicated_hydrogens,
+                        # P-25.3.3.1.2 (a): Lower locants for heteroatoms
+                        \&ChemOnomatopist::rule_lowest_numbered_heteroatoms,
+                        # P-25.3.3.1.2 (b): Lower locants for senior heteroatoms
+                        \&ChemOnomatopist::rule_lowest_numbered_most_senior_heteroatoms,
+                        # P-25.3.3.1.2 (c): Lower locants for fusion carbon atoms
+                        \&rule_lowest_numbered_fusion_carbons,
+                        # P-25.3.3.1.2 (d): Lower locants for fusion heteroatoms (rather than nonfusion)
+                        \&rule_lowest_numbered_fusion_heteroatoms,
+                        # TODO: P-25.3.3.1.2 (e): Lower locants for interior heteroatom
+                        # P-25.3.3.1.2 (f): Lower locants for indicated hydrogen atoms
+                        \&ChemOnomatopist::rule_lowest_numbered_indicated_hydrogens,
+
+                        # P-14.4 (f): Lower locants for detachable alphabetized prefixes
+                        # CHECKME: Not sure what is the relation with other rules
+                        # FIXME: Does not take alphabetic order in consideration
+                        \&ChemOnomatopist::rule_lowest_numbered_locants,
                      ) {
             my @candidates_now = $rule->( @candidates );
             if( @candidates_now == 1 ) {
@@ -194,7 +198,7 @@ sub new
             }
         }
 
-        $self->{vertices} = [ (shift @candidates)->vertices ]; # FIXME: Simply return instead of self
+        $self->vertices( (shift @candidates)->vertices ); # FIXME: Simply return instead of self
     } elsif( $nbenzene == 1 ) {
         # Numbering has to start from cycle other than benzene
         if( $cycles[0]->is_benzene ) {
@@ -209,7 +213,7 @@ sub new
             @cycles = map { $_->flipped } @cycles;
             $self->{cycles} = \@cycles;
         }
-        $self->_adjust_vertices_to_cycles; # CHECKME: Is this needed?
+        $self->_adjust_vertices_to_cycles;
     }
 
     return $self;
@@ -239,7 +243,7 @@ sub flipped_horizontally()
     my $copy = $self->copy;
     my @vertices = reverse $copy->vertices;
     push @vertices, shift @vertices;
-    $copy->{vertices} = \@vertices;
+    $copy->vertices( @vertices );
     return $copy;
 }
 
@@ -253,7 +257,7 @@ sub flipped_vertically()
     for (1..$cycle->length-2) {
         unshift @vertices, pop @vertices;
     }
-    $copy->{vertices} = \@vertices;
+    $copy->vertices( @vertices );
     return $copy;
 }
 
@@ -284,14 +288,14 @@ sub fusion_positions()
 sub fusion_carbon_positions()
 {
     my( $self ) = @_;
-    return grep { ChemOnomatopist::element( $self->{vertices}[$_] ) eq 'C' }
+    return grep { ChemOnomatopist::Util::element( $self->{vertices}[$_] ) eq 'C' }
                 $self->fusion_positions;
 }
 
 sub fusion_heteroatom_positions()
 {
     my( $self ) = @_;
-    return grep { ChemOnomatopist::element( $self->{vertices}[$_] ) ne 'C' }
+    return grep { ChemOnomatopist::Util::element( $self->{vertices}[$_] ) ne 'C' }
                 $self->fusion_positions;
 }
 
@@ -304,7 +308,7 @@ sub parent(;$)
 
     if( $self->is_naphthalene ) {
         my( $chain ) = ChemOnomatopist::filter_chains( $self->candidates );
-        $self->{vertices} = [ $chain->vertices ];
+        $self->vertices( $chain->vertices );
     }
 
     return $old_parent;
@@ -362,6 +366,13 @@ sub is_naphthalene()
     return $self->is_hydrocarbon && all { $_->length == 6 } $self->cycles;
 }
 
+sub is_naphthyridine()
+{
+    my( $self ) = @_;
+    return '' unless all { $_->length == 6 && $_->is_monoreplaced } $self->cycles;
+    return join( ',', uniq $self->heteroatoms ) eq 'N';
+}
+
 sub is_purine()
 {
     my( $self ) = @_;
@@ -386,7 +397,7 @@ sub needs_indicated_hydrogens() { 1 }
 sub needs_heteroatom_locants()
 {
     my( $self ) = @_;
-    return $self->suffix =~ /^benzo/;
+    return $self->suffix =~ /^benzo/ || $self->is_naphthyridine;
 }
 
 sub needs_heteroatom_names()
@@ -452,6 +463,10 @@ sub suffix()
         }
     }
 
+    if( $self->is_naphthyridine ) {
+        return ChemOnomatopist::Name::Part::Stem->new( 'naphthyridine' )->to_name;
+    }
+
     my @SMILES = map { $_->backbone_SMILES } $self->cycles;
     print STDERR "bicycle SMILES: @SMILES\n" if $ChemOnomatopist::DEBUG;
     my $retained = find_retained( @SMILES );
@@ -582,11 +597,12 @@ sub _adjust_vertices_to_cycles()
 
     my @cycles = $self->cycles;
 
-    $self->{vertices} = [];
-    push @{$self->{vertices}}, $cycles[0]->vertices;
-    pop  @{$self->{vertices}};
-    push @{$self->{vertices}}, $cycles[1]->vertices;
-    pop  @{$self->{vertices}};
+    my @vertices;
+    push @vertices, $cycles[0]->vertices;
+    pop  @vertices;
+    push @vertices, $cycles[1]->vertices;
+    pop  @vertices;
+    $self->vertices( @vertices );
 
     return $self;
 }
