@@ -8,7 +8,6 @@ use warnings;
 
 use parent ChemOnomatopist::Chain::Circular::;
 
-use ChemOnomatopist;
 use ChemOnomatopist::Elements qw( %elements );
 use ChemOnomatopist::Group::Sulfinyl;
 use ChemOnomatopist::Group::Sulfonyl;
@@ -79,13 +78,13 @@ sub new
     my( $senior_heteroatom ) = sort { $elements{$a}->{seniority} <=>
                                       $elements{$b}->{seniority} }
                                grep { $_ ne 'C' }
-                               map  { ChemOnomatopist::element( $_ ) }
+                               map  { ChemOnomatopist::Util::element( $_ ) }
                                     @vertices;
     return $self unless $senior_heteroatom;
 
     my @chains;
     for (circle_permutations( @vertices )) {
-        next unless ChemOnomatopist::element( $_->[0] ) eq $senior_heteroatom;
+        next unless ChemOnomatopist::Util::element( $_->[0] ) eq $senior_heteroatom;
         push @chains, ChemOnomatopist::Chain::Circular->new( $graph, @$_ );
     }
 
@@ -105,13 +104,13 @@ sub candidates()
     my @chains;
     for (0..$#vertices) {
         push @chains, bless { graph => $graph, vertices => [ @vertices ], parent => $parent }, ChemOnomatopist::Chain::Monocycle::
-            if !$senior_heteroatom || ChemOnomatopist::element( $vertices[0] ) eq $senior_heteroatom;
+            if !$senior_heteroatom || ChemOnomatopist::Util::element( $vertices[0] ) eq $senior_heteroatom;
         push @vertices, shift @vertices;
     }
     @vertices = reverse @vertices;
     for (0..$#vertices) {
         push @chains, bless { graph => $graph, vertices => [ @vertices ], parent => $parent }, ChemOnomatopist::Chain::Monocycle::
-            if !$senior_heteroatom || ChemOnomatopist::element( $vertices[0] ) eq $senior_heteroatom;
+            if !$senior_heteroatom || ChemOnomatopist::Util::element( $vertices[0] ) eq $senior_heteroatom;
         push @vertices, shift @vertices;
     }
 
@@ -129,7 +128,7 @@ sub autosymmetric_equivalents()
 {
     my( $self ) = @_;
 
-    my $cycle = $self->graph->subgraph( [ $self->vertices ] ); # TODO: Add attributes
+    my $cycle = $self->graph->subgraph( $self->vertices ); # TODO: Add attributes
     my @chains = map { ChemOnomatopist::Chain::Circular->new( $cycle, @$_ ) }
                      circle_permutations( $self->vertices );
     # CHECKME: Additional rules from ChemOnomatopist::filter_chains() might still be needed
@@ -151,7 +150,7 @@ sub parent(;$)
     @candidates = rule_lowest_parent_locant( @candidates ) if defined $self->parent_locant;
     @candidates = ChemOnomatopist::filter_chains( @candidates );
 
-    $self->{vertices} = [ $candidates[0]->vertices ];
+    $self->vertices( $candidates[0]->vertices );
     return $old_parent;
 }
 
@@ -160,14 +159,14 @@ sub needs_heteroatom_locants()
     my( $self ) = @_;
     return '' if $self->is_hydrocarbon;
     return '' if $self->is_monoreplaced && !$self->is_substituted;
-    return $self->length < 3 || $self->length > 10 || all { ChemOnomatopist::element( $_ ) ne 'C' } $self->vertices;
+    return $self->length < 3 || $self->length > 10 || all { ChemOnomatopist::Util::element( $_ ) ne 'C' } $self->vertices;
 }
 
 sub needs_heteroatom_names()
 {
     my( $self ) = @_;
     return '' if $self->is_hydrocarbon;
-    return $self->length < 3 || $self->length > 10 || all { ChemOnomatopist::element( $_ ) ne 'C' } $self->vertices;
+    return $self->length < 3 || $self->length > 10 || all { ChemOnomatopist::Util::element( $_ ) ne 'C' } $self->vertices;
 }
 
 sub needs_indicated_hydrogens()
@@ -262,7 +261,7 @@ sub suffix()
         my %heteroatoms;
         my @vertices = $self->vertices;
         for my $i (0..$#vertices) {
-            my $symbol = ChemOnomatopist::element( $vertices[$i] );
+            my $symbol = ChemOnomatopist::Util::element( $vertices[$i] );
             next if $symbol eq 'C';
             $heteroatoms{$symbol} = [] unless $heteroatoms{$symbol};
             push @{$heteroatoms{$symbol}}, $i;
