@@ -7,6 +7,7 @@ use strict;
 use warnings;
 
 use ChemOnomatopist::Util qw( copy );
+use Chemistry::OpenSMILES qw( is_double_bond );
 use Exporter;
 use Graph::Traversal::BFS;
 use List::Util qw( any sum0 );
@@ -27,6 +28,7 @@ our @EXPORT_OK = qw(
     graph_path_between_vertices
     graph_without_edge_attributes
     merge_graphs
+    neighbours_at_distance
     subgraph
     tree_branch_positions
     tree_number_of_branches
@@ -36,8 +38,7 @@ our @EXPORT_OK = qw(
 sub BFS_calculate_chain_length
 {
     my( $graph, $start ) = @_;
-    my $bfs = Graph::Traversal::BFS->new( $graph, start => $start );
-    return scalar $bfs->bfs;
+    return scalar Graph::Traversal::BFS->new( $graph, start => $start )->bfs;
 }
 
 # Returns 1 if there is any braches in the given graph and if there is none
@@ -45,7 +46,7 @@ sub BFS_is_chain_branched
 {
     my( $graph, $start ) = @_;
 
-    # FIXME: Not entirely sure why visited vertices are removed (A.M.)
+    # FIXME: Not entirely sure why visited vertices are removed
     my $graph_copy = $graph->copy;
 
     my $branched = 0;
@@ -70,7 +71,7 @@ sub cyclic_components
 
     # Due to the issue in Graph, bridges() returns strings instead of real objects.
     # Graph issue: https://github.com/graphviz-perl/Graph/issues/29
-    # The code below works now buggy (< 0.9727) as well as fixed (>= 0.9727) versions.
+    # The code below works on buggy (< 0.9727) as well as fixed (>= 0.9727) versions.
     my %vertices_by_name = map { $_ => $_ } $graph->vertices;
     $graph->delete_edges( map { map { $vertices_by_name{$_} } @$_ } $graph->bridges );
     $graph->delete_vertices( grep { !$graph->degree( $_ ) } $graph->vertices );
@@ -146,8 +147,8 @@ sub graph_cycles
         for my $component ($wo_chords->connected_components) {
             next if @$component == 1;
             my @ends = grep { $wo_chords->degree( $_ ) == 1 } @$component; # Should be two
-            my( $chord ) = grep { ($graph->has_edge( $_->[0], $ends[0] ) && $graph->has_edge( $_->[1], $ends[1] )) ||
-                                  ($graph->has_edge( $_->[0], $ends[1] ) && $graph->has_edge( $_->[1], $ends[0] )) } @chords;
+            my $chord = first { ($graph->has_edge( $_->[0], $ends[0] ) && $graph->has_edge( $_->[1], $ends[1] )) ||
+                                ($graph->has_edge( $_->[0], $ends[1] ) && $graph->has_edge( $_->[1], $ends[0] )) } @chords;
             next unless $chord;
             # Found a chord which completes a cycle
             my $vertices = set( @$component, @$chord );
@@ -247,10 +248,10 @@ sub graph_longest_paths_from_vertex
     for my $vertex ( @furthest_leaves ) {
         my @path;
         while( defined $vertex ) {
-            push @path, $vertex;
+            unshift @path, $vertex;
             $vertex = $from{$vertex};
         }
-        push @longest_paths, [ reverse @path ];
+        push @longest_paths, \@path;
     }
 
     return @longest_paths;
@@ -281,6 +282,24 @@ sub merge_graphs
     return $merged;
 }
 
+sub neighbours_at_distance
+{
+    my( $graph, $vertex, $from, $distance, $path ) = @_;
+
+    my @neighbours = grep { !$path->has( $_ ) }
+                          $graph->neighbours( $vertex );
+    my @double_bonds = grep { is_double_bond( $graph, $vertex, $_ ) }
+                            $graph->neighbours( $vertex );
+
+    if( $distance ) {
+        return @double_bonds,
+               map { neighbours_at_distance( $graph, $_, $vertex, $distance-1, set( @$path, $vertex ) ) }
+                   @neighbours;
+    } else {
+        return @double_bonds, @neighbours;
+    }
+}
+
 sub subgraph
 {
     my( $graph, @vertices ) = @_;
